#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""
Created on Tue Jul 11 18:23:51 2017

@author: hielke
"""

# This code is part of the software of iGEM TUDelft 2017 and governed by 
# its license.  Please see the LICENSE file that is also included.

from card_functions import getSubTerms, processGenes
from custom_functions import cd, mkdir, saveCat, SmartFormatter
import argparse

def findChildren(entry, LINKS, BASE): 
    """
    | From a given entry find the relevant children. 
    | These are 'confers_resistance_to' and 'confers_resistance_to_drug'
    | It will then recursively proceed to its children. 
    | If there are no children, then genes are processed and saved. 
    """
    name = mkdir(entry)
    with cd(name):
        subTerms = getSubTerms(entry, BASE)
        genes = saveCat(*list(map(subTerms.get, LINKS))) 
        if genes is not None:
            for gene in genes:
                processGenes(gene, BASE)
        
        children = subTerms.get('is_a')
        if children is not None:
            for child in children:
                findChildren(child, BASE)
        

if __name__ == '__main__':
    RELATIONS = ['confers_resistance_to', 'confers_resistance_to_drug']
    
    parser = argparse.ArgumentParser(description="Run the webcrawler on the meme database", 
                                     formatter_class=SmartFormatter)
    parser.add_argument('-b', '--base', default='https://card.mcmaster.ca', 
                        help="The location of the CARD database. DEFAULT='https://card.mcmaster.ca'")
    parser.add_argument('-l', '--link', default='/ontology/36008',
                        help="The link were to start searching. DEFAULT='/ontology/36008'")
    parser.add_argument('-n', '--name', default='antibiotic molecule', 
                        help="The name of the folder were everything will be saved. DEFAULT='antibiotic molecule'")
    parser.add_argument('relations', default=RELATIONS, 
                        help="The links which to follow for the recursive search. DEFAULT=" + str(RELATIONS), 
                        nargs=argparse.REMAINDER)
    args = parser.parse_args()
    entry = (args.name, args.link)
    print("Putting files in " + mkdir(entry))
    print("Connecting to " + args.base)
    print("Relating to " + str(args.relations))
    
    findChildren(entry, args.links, args.base)
    
    
# TRASH BELOW   
# import time
#if args.test:
#    entry = ('acridine dye', '/ontology/35963')
#    with cd('antibiotic_molecule_36008'):
#        findChildren(entry)
#
#
#if __name__ == '__main__' and test == True:
#    ts = time.time()
#    start = ('antibiotic molecule', '/ontology/36008')
#    findChildren(start)
#    te = time.time()
#    print('FINISHED: after a total of %d seconds.' % (te - ts))
#    
