#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""
Created on Sun Aug  6 17:03:37 2017

@author: hielke
"""

# This code is part of the software of iGEM TUDelft 2017 and governed by 
# its license.  Please see the LICENSE file that is also included.

# Pythonic modules
from subprocess import Popen
import os
import argparse

# My own modules
from custom_functions import cd, fileCat, sh, SmartFormatter
from meme_functions import memeCmd

PROCESS = 'gather' # ('gathering', 'meme', 'testing', meme_on_one)

def gather_all(root):
    with cd(root):
        for dire in os.listdir():
            with cd(dire):
                fileCat(".fasta", verbose=False)

def gather(dirs):
    for dire in dirs:            
        with cd(dire):
            fileCat('.fasta', verbose=True)

def meme(dirs, args):
    for dire in dirs:
        with cd(dire):
            # finding the correct file
            
            for f in sh('find -name "*%s"' % args.extension, verbose=False).splitlines():
                if 'temp' in f: # if it isn't the temporary file
                    continue
                file = f
            
            ran = range(args.begin, args.end, args.step)
            if args.motifs:
                cmds = [memeCmd(file=file, sites=i, motifs=args.motifs) 
                        for i in ran]
            else:
                num = len([file 
                        for file in sh('find -name "*.fasta"').splitlines() 
                        if 'gathered' not in file])
                cmds = [memeCmd(file=file, sites=i, 
                        motifs=(num//i if num//i<100 else 100)) 
                        for i in ran]
            print(cmds)
            processes = [Popen(cmd, shell=True) for cmd in cmds]
            print("We are running")
            for p in processes:
                p.wait()
            print("We are finished")
            
            
def meme_all(root, args):
    with cd(root):
        meme(os.listdir(), args)



if __name__ == '__main__':
    MODES = ['gather', 'meme']
    
    parser = argparse.ArgumentParser(description="What kind of process related to meme you " 
                                     + "want to run", formatter_class=SmartFormatter)
    parser.add_argument("mode", help="R|The mode you want to run in. Choose from: " 
                        + str(MODES) + "\n\n"
                        + "gather: Creates one fastafile from all fastafiles.\n"
                        + "    Not all the options are used for this one.\n\n"
                        + "meme: Start the meme process with the folowing options:\n\n")
    parser.add_argument('-b', '--begin', default=2, type=int, 
                        help="For the range of the amount of sites to visit, " 
                        + "this is the beginning. (Including) DEFAULT=2")
    parser.add_argument('-e', '--end', default=7, type=int, 
                        help="For the range of the amount of sites to visit, " 
                        + "this is the end. (Excluding) DEFAULT=7")
    parser.add_argument('-s', '--step', default=1, type=int, help="The step size " 
                        + "for the range of the amount of sites to visit. DEFAULT=1")
    parser.add_argument('-m', '--motifs', type=int, help="The amount "
                        + "of motifs you want to find, otherwise a bit of a " 
                        + "clever algorithm tries to find these.")
    parser.add_argument('-x', '--extension', default=".multiple.fasta", 
                        help="The extension of the file that contains all fasta "
                        + " sequences, it doesn't allows temp in the filename. DEFAULT=.multiple.fasta")
    parser.add_argument('-r', '--root', help="Give the root folder in which the " 
                        + "process has to work on all available directories, "
                        + "if you want to do the same " 
                        + "process in a given set of directories, provide them at the " 
                        + "end of the command. and leave this one unused.") # the dirs, see next:
    parser.add_argument('dirs', nargs=argparse.REMAINDER , 
                        help="Optional different folders in which to execute " 
                        + "the same process")
    args = parser.parse_args()
    
    # adjust MODES if you add a mode. 
    
    if args.mode == 'gather':
        if args.root:
            print("Gather all in " + args.root)
            gather_all(root=args.root)
        else:
            print("Gather in " + str(args.dirs))
            gather(args.dirs)
    elif args.mode == 'meme':
        if args.root:
            print("Meme on all in " + args.root)
            if args.motifs:
                print("With the options: " 
                  + "range(%i, %i, %i) " % (args.begin, args.end, args.step)
                  + "and find %i motifs with the file ending on " % (args.motifs)
                  + args.extension)
            else:
                print("With the options: " 
                  + "range(%i, %i, %i) " % (args.begin, args.end, args.step)
                  + "and find a good amount of motifs per directory with the "
                  + "file ending "
                  + args.extension)
            meme_all(root=args.root, args=args)
        else:
            print("Meme in " + str(args.dirs))
            if args.motifs:
                print("With the options: " 
                  + "range(%i, %i, %i) " % (args.begin, args.end, args.step)
                  + "and find %i motifs with the file ending on " % (args.motifs)
                  + args.extension)
            else:
                print("With the options: " 
                  + "range(%i, %i, %i) " % (args.begin, args.end, args.step)
                  + "and find a good amount of motifs per directory with the "
                  + "file ending on "
                  + args.extension)
            meme(args.dirs, args)            
    else:
        print("[ERROR]: No such mode, choose from: " + str(MODES))
        
        
# TRASH BELOW
#
#def hoi():
#    print(PROCESS)
#                
#
#if PROCESS == 'testing':
#    with cd('/home/hielke/py/antibiotic_molecule_36008'):
#        print(os.listdir())
#    hoi()
#    with cd('test'):
#       print(len([file 
#                  for file in sh('find . -name "*.fasta"').splitlines() 
#                          if 'gathered' not in file]))
#    inputvalues = (4, 3)
#    arraysize = (3, 3)  
#    lijst = [a + 4*b for a, b in zip((3,4), (7, 8))]
#    print(lijst)
#    assert all([inp <= size for inp, size in zip(inputvalues, arraysize)]), \
#            "Your input values are too large"
#
#"""['fluoroquinolone_antibiotic_35920', 'beta-lactam_antibiotic_36016', 'aminocoumarin_antibiotic_36242', 'glycopeptide_antibiotic_36220', 'phenicol_antibiotic_36526', 'lincosamide_antibiotic_35936', 'macrolide_antibiotic_35919']
#['polyamine_antibiotic_36666' xxx
# 'peptide_antibiotic_36192' xxx
# 'macrolide_antibiotic_35919'  xxx
# 'lipopeptide_antibiotic_36175' xxx
# 'aminoglycoside_antibiotic_35935' xxx
# 'sulfonamide_antibiotic_36421' xxx
# 'rifamycin_antibiotic_36296' xxx
# 'acridine_dye_36193' xxx
# 'macrocyclic_antibiotic_37717' xxx
# 'antibiotic_mixture_37087' xxx
# 'pleuromutilin_antibiotic_37014' xxx
# 'tetracycline_35968' xxx
# 'antibacterial_free_fatty_acids_40727' xxx
# 'phenicol_antibiotic_36526' xxx
# 'elfamycin_antibiotic_37618' xxx
# 'fluoroquinolone_antibiotic_35920' xxx
# 'glycopeptide_antibiotic_36220' xxx
# 'diaminopyrimidine_antibiotic_36310' xxx
# 'beta-lactam_antibiotic_36016' x (only 20 percent)
# 'miscellaneous_antibiotic_36012' xxx
# 'aminocoumarin_antibiotic_36242' xxx
# 'streptogramin_antibiotic_35945' xxx
# 'nucleoside_antibiotic_36174' xxx
#* 'oxazolidinone_antibiotic_36218' xxx
# 'tetracycline_derivative_36189'
# 'sulfone_39985' xxx
# 'lincosamide_antibiotic_35936' xxx
# 'organoarsenic_antibiotic_36690' xxx
# ]"""    
#
#if PROCESS == 'meme_on_one':
#    with cd('mecA'):
#        num = len([file 
#                      for file in sh('find -name "*.fasta"').splitlines() 
#                              if 'gathered' not in file])
#        for f in sh('find -name "*.multiple.fasta"', verbose=False).splitlines():
#            if 'temp' in f: # if it is the temporary file
#                continue
#            cmds = [memeCmd(file=f, sites=i, motifs=30) for i in range(2, 7)]
#            print(cmds)
#            cmd_groups = grouper(cmds, 3, fillvalue=None)
#            for group in cmd_groups:
#                processes = [Popen(cmd, shell=True) for cmd in group]
#                for p in processes:
#                    p.wait()
#                print("Going to the next")
#            print("We are finished")
#        
#
#if PROCESS == 'meme':
#    extension = '.multiple.fasta'
#    ran = range(11,14)
#    cds = ['aminocoumarin_antibiotic_362 ', 'phenicol_antibiotic_36526', 'lincosamide_antibiotic_35936', 'macrolide_antibiotic_35919']
#    cds = [ 'tetracycline_derivative_36189']
#    cds = ['streptogramin_antibiotic_35945']
#    cds = [
#    'fluoroquinolone_antibiotic_35920',
#    'aminoglycoside_antibiotic_35935'
#            ]
#    for c in cds:
#        root = 'antibiotic_molecule_36008'
#        with cd(root + '/'+ c):
#            num = len([file 
#                      for file in sh('find -name "*.fasta"').splitlines() 
#                              if 'gathered' not in file])
#            for f in sh('find -name "*%s"' % extension, verbose=False).splitlines():
#                if 'temp' not in f: # if it isn't the temporary file
#                    cmds = [memeCmd(f, i, num//i if num//i<100 else 100) for i in ran]
#                    print(cmds)
#                    processes = [Popen(cmd, shell=True) for cmd in cmds]
#                    print("We are running")
#                    for p in processes:
#                        p.wait()
#                    print("We are finished")        
